/* foXcontrol-desktop
 * Copyright (C) 2000-2001 Josua Groeger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "FDCDesktopColours.h"
#include "ctype.h"

FXDEFMAP (FDCDesktopColours) FDCDesktopColoursMap[]=
{
  FXMAPFUNC (SEL_COMMAND, FDCDesktopColours::ID_SCHEME_COMBO   , FDCDesktopColours::onSchemeCombo),
  FXMAPFUNC (SEL_COMMAND, FDCDesktopColours::ID_SCHEME_CREATE  , FDCDesktopColours::onSchemeCreate),
  FXMAPFUNC (SEL_COMMAND, FDCDesktopColours::ID_SCHEME_REMOVE  , FDCDesktopColours::onSchemeRemove),
  FXMAPFUNC (SEL_COMMAND, FDCDesktopColours::ID_SCHEME_SELECTOR, FDCDesktopColours::onSchemeSelector)
};

FDCIMPLEMENT (FDCDesktopColours, FDComponent, FDCDesktopColoursMap, ARRAYNUMBER (FDCDesktopColoursMap));

FDCDesktopColours::FDCDesktopColours (FDCARGS): FDComponent ("foXcontrol", "DesktopColours", FDCPARAMS)
{
  FXVerticalFrame *contentsFrame= new FXVerticalFrame (this, LAYOUT_CENTER_X|LAYOUT_CENTER_Y| \
                                                       PACK_UNIFORM_WIDTH, 0,0,0,0,0,0,0,0);
  
  // Colour schemes
  FXGroupBox *schemeBox= new FXGroupBox (contentsFrame, "Colour schemes", GROUPBOX_TITLE_LEFT|FRAME_THICK);
  FXHorizontalFrame *schemeFrame= new FXHorizontalFrame (schemeBox, LAYOUT_FILL_X);
  schemeCombo= new FXComboBox (schemeFrame, 10, 10, this, ID_SCHEME_COMBO,
                               FRAME_NORMAL|LAYOUT_CENTER_Y|COMBOBOX_STATIC);

  FXHorizontalFrame *buttonFrame= new FXHorizontalFrame (schemeFrame, LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  new FXButton (buttonFrame, "Cr&eate", NULL, this, ID_SCHEME_CREATE); 
  removeButton= new FXButton (buttonFrame, "Re&move", NULL, this, ID_SCHEME_REMOVE);

  // Colours
  FXGroupBox *colorBox= new FXGroupBox (contentsFrame, "Common colours", GROUPBOX_TITLE_LEFT|FRAME_THICK);  
  schemeSelector= new FDColScmSelector (colorBox, DEFAULT_CS, this, ID_SCHEME_SELECTOR, LAYOUT_FILL_X);

  registry       = new FDColScmRegistry (reg ().getAppKey (), reg ().getVendorKey ());
  desktopRegistry= new FDColScmRegistry ("Desktop");
}

FDCDesktopColours::~FDCDesktopColours ()
{
  for (FXuint i= 0; i< schemeCombo->getNumItems (); i++)
    delete (FDColorScheme*) schemeCombo->getItemData (i);

  delete registry;
  delete desktopRegistry;
}

// Events --------------------------------------------------------------------------------------------

long FDCDesktopColours::onSchemeCombo (FXObject*, FXSelector sel, void*)
{
  // Jump to the next item if this is just a "caption"
  while (schemeCombo->getText ()[0]== '>') (schemeCombo->getCurrentItem ()== schemeCombo->getNumItems ()-1)?
    schemeCombo->setCurrentItem (0): // if last item, jump to the first one
    schemeCombo->setCurrentItem (schemeCombo->getCurrentItem ()+1); // jump to the next one

  // Disable the remove button if the colour scheme is a system or a special one
  (reg ().readIntEntry ("SYSTEM COLOR SCHEMES", schemeCombo->getText ().text (), 0) ||
   schemeCombo->getText ()[0]== '(')? removeButton->disable (): removeButton->enable ();

  // Set the selector's colour scheme
  for (FXint i= 0; i< schemeCombo->getNumItems (); i++)
    if (schemeCombo->getText ()== schemeCombo->getItemText (i))
    {
      schemeSelector->setColorScheme (*(FDColorScheme*) schemeCombo->getItemData (i));
      return 1;
    }

  return 0;
}

long FDCDesktopColours::onSchemeCreate (FXObject*, FXSelector sel, void*)
{
  FXString result; if (schemeCombo->getText ()[0]!= '(') result= schemeCombo->getText ();
  if (!FXInputDialog::getString (result, this, "Create a colour scheme",
                                 "Add a new or modify an existing colour scheme:",
                                 ((FDApp*) getApp ())->ico ().insert ("foXcolours.xpm"))) return 0;

  // Empty?
  if (result.empty ())
  {
    FXMessageBox::error (this, MBOX_OK, "Create a colour scheme", "The colour scheme name is empty!");
    return 0;
  }

  // Illegal characters?
  for (FXuint i= 0; result[i]; i++) if(!isalnum (result[i]) && result[i]!= '_' && result[i]!= '-' &&
                                       result[i]!= '.' && result[i]!= ' ' && result[i]!= PATHSEP)
  {
    FXMessageBox::error (this, MBOX_OK, "Create a colour scheme",
                         "The colour scheme name contains one or more illegal character(s)!");
    return 0;
  }

  // System colour scheme?
  if (reg ().readIntEntry ("SYSTEM COLOR SCHEMES", result.text (), 0))
  {
    FXMessageBox::error (this, MBOX_OK, "Create a colour scheme",
                         "System colour schemes may not be overwritten!");
    return 0;
  }

  // Modify an existing colour scheme
  for (FXint i= 0; i< schemeCombo->getNumItems (); i++) if (result== schemeCombo->getItemText (i))
  {
    *(FDColorScheme*) schemeCombo->getItemData (i)= schemeSelector->getColorScheme ();
    schemeCombo->setCurrentItem (i);
    handle (this, MKUINT (ID_SCHEME_COMBO, SEL_COMMAND), NULL);
    return 1;
  }

  // Add a new colour scheme
  schemeCombo->appendItem (result, new FDColorScheme (schemeSelector->getColorScheme ()));
  schemeCombo->setCurrentItem (schemeCombo->getNumItems ()-1);
  handle (this, MKUINT (ID_SCHEME_COMBO, SEL_COMMAND), NULL);

  return 1;
}

long FDCDesktopColours::onSchemeRemove (FXObject*, FXSelector sel, void*)
{
  for (FXint i= 0; i< schemeCombo->getNumItems (); i++)
    if (schemeCombo->getText ()== schemeCombo->getItemText (i))
  {
    delete (FDColorScheme*) schemeCombo->getItemData (i);
    schemeCombo->removeItem (i);
    handle (this, MKUINT (ID_SCHEME_COMBO, SEL_COMMAND), NULL);
    return 1;
  }
  return 0;
}

long FDCDesktopColours::onSchemeSelector (FXObject*, FXSelector sel, void*)
{
  schemeCombo->setText ("(New)");
  removeButton->disable ();
  return 1;
}

// Virtual methods ------------------------------------------------------------------------------------

void FDCDesktopColours::read ()
{
  registry->read ();
  desktopRegistry->read ();

  // Clear schemeCombo
  for (FXuint i= 0; i< schemeCombo->getNumItems (); i++)
    delete (FDColorScheme*) schemeCombo->getItemData (i);
  schemeCombo->clearItems ();
 
  schemeCombo->appendItem (">>> User <<<");

  // Add system and user colour schemes to schemeCombo 
  FXStringDict *group= registry->getSections().find ("COLOR SCHEMES");
  if (group)
  {
    FDColorScheme *tmpScheme;
    FXint pos= group->last ();
    while (pos>= 0)
    {
      if (group->data (pos)[0] && group->data (pos)[0]!= '(' && group->data (pos)[0]!= '>')
      {
        tmpScheme= new FDColorScheme (registry->readColorSchemeEntry ("COLOR SCHEMES", group->key (pos)));
        
        // System colour scheme?
        (registry->readIntEntry ("SYSTEM COLOR SCHEMES", group->key (pos), 0))?
          schemeCombo->prependItem (group->key (pos), tmpScheme): // Prepend system colour schemes
          schemeCombo->appendItem (group->key (pos), tmpScheme);  // Append user colour schemes
      }
	  
      pos= group->prev (pos);
    }
  }

  // Add the first three entries
  schemeCombo->prependItem (">>> System <<<");
  schemeCombo->prependItem ("(Default)", new FDColorScheme (DEFAULT_CS));
  schemeCombo->prependItem ("(Current)", new FDColorScheme (desktopRegistry->readColorSchemeEntry ()));

  // Make "(Current)" schemeCombo's current item
  schemeCombo->setCurrentItem (0);
  handle (this, MKUINT (ID_SCHEME_COMBO, SEL_COMMAND), NULL);
}

void FDCDesktopColours::write ()
{
  desktopRegistry->writeColorSchemeEntry (schemeSelector->getColorScheme ());  

  registry->deleteSection ("COLOR SCHEMES");
  for (FXuint i= schemeCombo->getNumItems ()-1; schemeCombo->getItemText (i)[0]!= '>'; i--)
    registry->writeColorSchemeEntry ("COLOR SCHEMES", schemeCombo->getItemText (i).text (),
                                                      *(FDColorScheme*) schemeCombo->getItemData (i));
  registry->write ();
  desktopRegistry->write ();

  // Change current item
  *(FDColorScheme*) schemeCombo->getItemData (0)= schemeSelector->getColorScheme ();
  schemeCombo->setCurrentItem (0);
  handle (this, MKUINT (ID_SCHEME_COMBO, SEL_COMMAND), NULL);
}

void FDCDesktopColours::defaults ()
{
  schemeCombo->setCurrentItem (1);
  handle (this, MKUINT (ID_SCHEME_COMBO, SEL_COMMAND), NULL);
}

void FDCDesktopColours::about ()
{
  ((FDApp*) getApp ())->showAbout (this, "foXcontrol-desktop", VERSION, "Desktop/Colours", "foXcolours.xpm",
                                   "(C) 2000-2001 Josua Groeger <fractalus@gmx.net>",
                                   "http://foxdesktop.sourceforge.net");
}

void FDCDesktopColours::help ()
{
  ((FDApp*) getApp ())->showHelp ("foXcontrol-desktop", "colours.html");
}
