/* foXcontrol-desktop
 * Copyright (C) 2000-2001 Josua Groeger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "FDColScmRegistry.h"

FXbool FDColScmRegistry::writeColorSchemeEntry (const FXchar *section, const FXchar *key, FDColorScheme val)
{
  if (!section) {fxerror ("FDColScmRegistry::writeColorSchemeEntry: NULL section argument.\n");}
  if (!key)     {fxerror ("FDColScmRegistry::writeColorSchemeEntry: NULL key argument.\n");}
  FXStringDict *group= sections.insert (section);
  if (group)
  {  
    FXchar buffer[500];
    group->replace (key, fdnamefromcolorscheme (buffer, val), TRUE);

    modified=TRUE;
    return TRUE;
  }
  return FALSE;
};

FDColorScheme FDColScmRegistry::readColorSchemeEntry (const FXchar *section, const FXchar *key,
                                                      FDColorScheme def= DEFAULT_CS)
{
  if (!section) {fxerror ("FDColScmRegistry::readColorSchemeEntry: NULL section argument.\n");}
  if (!key)     {fxerror ("FDColScmRegistry::readColorSchemeEntry: NULL key argument.\n");}
  FXStringDict *group= sections.find (section);
  if (group)
  {
    const char *value= group->find (key);
    if (value) return fdcolorschemefromname (value, def);
  }
  return def;
}

FXbool FDColScmRegistry::writeColorSchemeEntry (FDColorScheme val)
{
  writeColorEntry ("SETTINGS", "bordercolor" , val[CBORDER]);
  writeColorEntry ("SETTINGS", "basecolor"   , val[CBASE]);
  writeColorEntry ("SETTINGS", "hilitecolor" , val[CHILITE]);
  writeColorEntry ("SETTINGS", "shadowcolor" , val[CSHADOW]);
  writeColorEntry ("SETTINGS", "backcolor"   , val[CBACK]);
  writeColorEntry ("SETTINGS", "forecolor"   , val[CFORE]);
  writeColorEntry ("SETTINGS", "selforecolor", val[CSELFORE]);
  writeColorEntry ("SETTINGS", "selbackcolor", val[CSELBACK]);
  
  modified= TRUE;
  return TRUE;
}

FDColorScheme FDColScmRegistry::readColorSchemeEntry (FDColorScheme def= DEFAULT_CS)
{
  FDColorScheme current_cs=
  {
    readColorEntry ("SETTINGS", "bordercolor" , def[CBORDER]),
    readColorEntry ("SETTINGS", "basecolor"   , def[CBASE]),
    readColorEntry ("SETTINGS", "hilitecolor" , def[CHILITE]),
    readColorEntry ("SETTINGS", "shadowcolor" , def[CSHADOW]),
    readColorEntry ("SETTINGS", "backcolor"   , def[CBACK]),
    readColorEntry ("SETTINGS", "forecolor"   , def[CFORE]),
    readColorEntry ("SETTINGS", "selforecolor", def[CSELFORE]),
    readColorEntry ("SETTINGS", "selbackcolor", def[CSELBACK])
  };

  return current_cs;
}
