/* foXcontrol-desktop
 * Copyright (C) 2000-2001 Josua Groeger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "FDColScmSelector.h"

FXIMPLEMENT (FDColScmSelector, FXPacker, NULL, 0)

FDColScmSelector::FDColScmSelector (FXComposite *p, FDColorScheme cs= DEFAULT_CS,
                                    FXObject *tgt= NULL, FXSelector sel=0, FXuint opts=0,
                                    FXint x=0, FXint y=0, FXint w=0, FXint h=0,
                                    FXint pl= DEFAULT_SPACING, FXint pr= DEFAULT_SPACING,
                                    FXint pt= DEFAULT_SPACING, FXint pb= DEFAULT_SPACING):
                  FXPacker (p, opts, x,y,w,h, pl,pr,pt,pb)
{
  FXHorizontalFrame *contentsFrame= new FXHorizontalFrame (this, LAYOUT_FILL_X|PACK_UNIFORM_HEIGHT,
                                                           0,0,0,0, 0,0,0,0, 0,0);

  FXVerticalFrame *labelFrame= new FXVerticalFrame (contentsFrame, 0, 0,0,0,0, 0,0,0,0, 0,0);
  new FXLabel (labelFrame, "Border colour"                    , NULL, LAYOUT_CENTER_Y);
  new FXLabel (labelFrame, "Background colour of GUI controls", NULL, LAYOUT_CENTER_Y);
  new FXLabel (labelFrame, "Highlight colour of GUI controls" , NULL, LAYOUT_CENTER_Y);
  new FXLabel (labelFrame, "Shadow colour of GUI controls"    , NULL, LAYOUT_CENTER_Y);
  new FXLabel (labelFrame, "Background colour"                , NULL, LAYOUT_CENTER_Y);
  new FXLabel (labelFrame, "Foreground colour"                , NULL, LAYOUT_CENTER_Y);
  new FXLabel (labelFrame, "Select foreground colour"         , NULL, LAYOUT_CENTER_Y);
  new FXLabel (labelFrame, "Select background colour"         , NULL, LAYOUT_CENTER_Y);
  
  FXVerticalFrame *wellFrame= new FXVerticalFrame (contentsFrame, LAYOUT_RIGHT, 0,0,0,0, 0,0,0,0, 0,0);
  borderWell = new FXColorWell (wellFrame, cs[CBORDER], tgt, sel, LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|
                                LAYOUT_FIX_HEIGHT|COLORWELL_OPAQUEONLY, 0, 0, 40, 25, 0,0,0,0);
  baseWell   = new FXColorWell (wellFrame, cs[CBASE], tgt, sel, LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|
                                LAYOUT_FIX_HEIGHT|COLORWELL_OPAQUEONLY, 0, 0, 40, 25, 0,0,0,0);
  hiliteWell = new FXColorWell (wellFrame, cs[CHILITE], tgt, sel, LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|
                                LAYOUT_FIX_HEIGHT|COLORWELL_OPAQUEONLY, 0, 0, 40, 25, 0,0,0,0);
  shadowWell = new FXColorWell (wellFrame, cs[CSHADOW], tgt, sel, LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|
                                LAYOUT_FIX_HEIGHT|COLORWELL_OPAQUEONLY, 0, 0, 40, 25, 0,0,0,0);
  backWell   = new FXColorWell (wellFrame, cs[CBACK], tgt, sel, LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|
                                LAYOUT_FIX_HEIGHT|COLORWELL_OPAQUEONLY, 0, 0, 40, 25, 0,0,0,0);
  foreWell   = new FXColorWell (wellFrame, cs[CFORE], tgt, sel, LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|
                                LAYOUT_FIX_HEIGHT|COLORWELL_OPAQUEONLY, 0, 0, 40, 25, 0,0,0,0);
  selforeWell= new FXColorWell (wellFrame, cs[CSELFORE], tgt, sel, LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|
                                LAYOUT_FIX_HEIGHT|COLORWELL_OPAQUEONLY, 0, 0, 40, 25, 0,0,0,0);
  selbackWell= new FXColorWell (wellFrame, cs[CSELBACK], tgt, sel, LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|
                                LAYOUT_FIX_HEIGHT|COLORWELL_OPAQUEONLY, 0, 0, 40, 25, 0,0,0,0);
}

FDColorScheme FDColScmSelector::getColorScheme ()
{
  FDColorScheme cs= {borderWell->getRGBA  (), baseWell->getRGBA (),
                     hiliteWell->getRGBA  (), shadowWell->getRGBA (),
                     backWell->getRGBA    (), foreWell->getRGBA (),
                     selforeWell->getRGBA (), selbackWell->getRGBA ()};
  return cs;
}

void FDColScmSelector::setColorScheme (FDColorScheme val)
{
  borderWell->setRGBA  (val[CBORDER]);
  baseWell->setRGBA    (val[CBASE]);
  hiliteWell->setRGBA  (val[CHILITE]);
  shadowWell->setRGBA  (val[CSHADOW]);
  backWell->setRGBA    (val[CBACK]);
  foreWell->setRGBA    (val[CFORE]);
  selforeWell->setRGBA (val[CSELFORE]);
  selbackWell->setRGBA (val[CSELBACK]);
}
