/* foXcontrol-desktop
 * Copyright (C) 2000-2001 Josua Groeger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "FDCDesktopFonts.h"

FXDEFMAP (FDCDesktopFonts) FDCDesktopFontsMap[]=
{
  FXMAPFUNC (SEL_COMMAND, FDCDesktopFonts::ID_CHANGE, FDCDesktopFonts::onChange)
};

FDCIMPLEMENT (FDCDesktopFonts, FDComponent, FDCDesktopFontsMap, ARRAYNUMBER (FDCDesktopFontsMap));

FDCDesktopFonts::FDCDesktopFonts (FDCARGS): FDComponent ("foXcontrol", "DesktopFonts", FDCPARAMS)
{
  FXGroupBox *groupBox= new FXGroupBox (this, "Normal font", LAYOUT_CENTER_X|LAYOUT_CENTER_Y| \
                                        GROUPBOX_TITLE_LEFT|FRAME_THICK);
  
  sampleLabel= new FXLabel (new FXPacker (groupBox, LAYOUT_FILL_X),
                            "ABCDEFGHIJKLMNOPQRSTUVWXYZ\nabcdefghijklmnopqrstuvwxyz\n0123456789",
                            NULL, LAYOUT_FILL_X|LAYOUT_CENTER_X|FRAME_SUNKEN);
  
  FXHorizontalFrame *changeFrame= new FXHorizontalFrame (groupBox, LAYOUT_FILL_X);  
  new FXButton (changeFrame, "C&hange", NULL, this, ID_CHANGE, LAYOUT_CENTER_Y|FRAME_RAISED|FRAME_THICK);
  changeLabel= new FXLabel (changeFrame, "", NULL, LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  
  fontDialog= new FXFontDialog (this, "Change normal font", DECOR_BORDER|DECOR_TITLE);
  sampleFont= NULL;
  desktopRegistry= new FXRegistry ("Desktop");
}

FDCDesktopFonts::~FDCDesktopFonts ()
{
  if (sampleFont) delete sampleFont;
  delete desktopRegistry;
}

void FDCDesktopFonts::setSampleFont (FXFontDesc &fontDesc)
{
  if (sampleFont) delete sampleFont;
  sampleFont= new FXFont (getApp(), fontDesc);
  sampleFont->create ();
  sampleLabel->setFont (sampleFont);
}

long FDCDesktopFonts::onChange (FXObject*, FXSelector, void*)
{
  FXFontDesc fontDesc;
  char fontSpec[200];

  fxparsefontdesc (fontDesc, changeLabel->getText ().text ());
  fontDialog->setFontSelection (fontDesc);
  
  if (fontDialog->execute ())
  {
    fontDialog->getFontSelection (fontDesc);
    fxunparsefontdesc (fontSpec, fontDesc);
    changeLabel->setText (fontSpec);
    setSampleFont (fontDesc);

    return 1;
  }

  return 0;
}

void FDCDesktopFonts::read ()
{
  desktopRegistry->read ();
  changeLabel->setText (desktopRegistry->readStringEntry ("SETTINGS","normalfont", "[helvetica] 90 700 1 0 0 0"));
  
  FXFontDesc fontDesc;
  fxparsefontdesc (fontDesc, changeLabel->getText ().text ());
  setSampleFont (fontDesc);
}

void FDCDesktopFonts::write ()
{
  desktopRegistry->writeStringEntry ("SETTINGS", "normalfont", changeLabel->getText ().text ());
  desktopRegistry->write ();
}

void FDCDesktopFonts::defaults ()
{
  changeLabel->setText ("[helvetica] 90 700 1 0 0 0");
  
  FXFontDesc fontDesc;
  fxparsefontdesc (fontDesc, changeLabel->getText ().text ());
  setSampleFont (fontDesc);
}

void FDCDesktopFonts::about ()
{
  ((FDApp*) getApp ())->showAbout (this, "foXcontrol-desktop", VERSION, "Desktop/Fonts", "foXfonts.xpm",
                                   "(C) 2000-2001 Josua Groeger <fractalus@gmx.net>",
                                   "http://foxdesktop.sourceforge.net");
}

void FDCDesktopFonts::help ()
{
  ((FDApp*) getApp ())->showHelp ("foXcontrol-desktop", "fonts.html");
}
